// 
// IR REMOTE TRANSMITTER PIC12F1822@CPS VERSION
//	ver1.0  Apr.27th,2013 (c)CyberPit HILO
//	
// Note:
//		This is Main Function Microchip XC8 C Language Source Code
//		Also need to link with "transmit.as" Assembler Source
//		which was inspired by following people
//			http://www.interline.or.jp/%7Etomcat/index2.html
//			http://phys.sci.hokudai.ac.jp/LABS/yts/tsujimi/tsujimi.html
//		Regardsing them, I also keep this code as freeware for the DIY freaks.
//												Thank you!	
//
#include<xc.h>
#include<pic12f1822.h>


#define	NOFCH	4		// Numbers of CPS channels (1 to 4)
#ifndef _XTAL_FREQ
 #define _XTAL_FREQ 4000000	// Compensated value for 8MHz CPU
#endif

/*
----< Configuration PIC Regs >---
*/
__CONFIG(FOSC_INTOSC & WDTE_OFF & PWRTE_ON & MCLRE_OFF & CP_OFF & CPD_OFF & BOREN_OFF & CLKOUTEN_OFF & IESO_OFF & FCMEN_OFF);
__CONFIG(WRT_OFF & PLLEN_OFF & STVREN_ON &  LVP_OFF);
                        
/*
----< Global Variabales  for INT process>----
*/                            
unsigned long SUM_LONG[4];	// Longtime moving Average Count
unsigned int SUM_SHORT[4];	// Shorttime moving average Count
unsigned char scanning_cps;	// Current IRQ processing CPS channel
unsigned char current_ch;	// Main function processing channel


/*
----< Global Variabales  for Assembler part>----
*/                            
unsigned char tx_data[8];
unsigned char ScnStatus;//40 kHz: set 0th(LSB) bit to 1 as 0bxxxxxx01
						//33 kHz: set 1st bit to 1 as 0bxxxxxx10
						//38 kHz: clear 0th and 1st bits to 0 as 0bxxxxxx00
						//sharp mode: set 2nd bit to 1
						//repeat feature will enable if 3rd bit set
						//numbers of repeat times defines upper 4 nibbles (0000 as Non Limit)
unsigned char rmcbyt;	// Bytes for sending data
unsigned char rmcbit;	// Additionl bits for sending data
unsigned char start_h;	// leader on time
unsigned char start_l;	// leader off time
unsigned char data_0h;	// data 0 on time
unsigned char data_1h;	// data 1 on time
unsigned char data_0l;	// data 0 off time
unsigned char data_1l;	// data 1 off time
unsigned char stop_h;	// stop on time
unsigned char rptwait_bf;	//repeat wait time for sharp mode
unsigned char rptwait;		//repeat wait time

extern void transmit(void);// assembly routine prototype description


/*
----< IR FORMAT SELECTION (ALPINE) >----
*/
void Alpine4byte(void){
	ScnStatus=0x00;//38kHz: Toshiba mode@(Each@Value*0.1mSec)
	rmcbyt=4;		//f[^̑oCg	
	rmcbit=0; 		//[rbg
	start_h=90; 	//X^[gM  ---- A
	start_l=42; 	//X^[gM   --- B
	data_0h=5; 		//f[^OM   --- C
	data_0l=6; 		//f[^OM   --- E
	data_1h=5; 		//f[^PM   --- D
	data_1l=17; 	//f[^PM   --- F
	stop_h=7;		//XgbvM   --- G
	rptwait_bf=0; 
	rptwait=0;		//s[g҂ (1 = 10ms)- H
}

/*
----< IR FORMAT SELECTION (PIONEER) >----
*/
void Carrozzeria4byte(void){
	ScnStatus=0x01;//40kHz:
	rmcbyt=4;		//f[^̑oCg	
	rmcbit=0; 		//[rbg
	start_h=84;  	//X^[gM  ---- A
	start_l=43;  	//X^[gM   --- B
	data_0h=5; 
	data_0l=5; 
	data_1h=5; 
	data_1l=16; 
	stop_h=7;
	rptwait_bf=0; 
	rptwait=9;
}

/*
----< MAIN >----
*/
void main(void)
{
 OSCCON=0b01101100;	//Use internal RC oscillator with  4MHz CPU clock
 WPUA5 =  0b00001000;	// Weak pull-up the RA3 port for the future use

 /* Setup mTouch Module*/
 CPSCON0=	0b00001100;	// Set CPS module OSC is High Range, Timer1 for CPS
 CPSCON1=	0b00000001;	// Select the channel of CPS

 /* Setup Ports*/	
 ANSELA = 	0b00010111;	// Define RA0/1/2/4 is CPS port as analog (==1) 
 TRISA 	= 	0b00011111;	// Define RA0/1/2/4 is CPS port as trisate (==1)

 /* Setup Timers and Counters */
 T1CON = 	0b11000001;	// Timer1 counts CPS Module's Clock
 T1GCON =	0b11000001;	// Enable Timer1, Active H, Toggle, Overflow 
 TMR1GE =	0;			// Disable Timer1 Gate Feature
 TMR0 =		0;			// Reset Timer0
 OPTION_REG=0b10000011;	// Set Timer0 overflow ocurrs every 2.048mS (Fosc/(4*8))

 INTCON =	0b00100000;	// O[o荞ݒ~Aӊ荞݋ATimer0@OVF荞݋


/* Pre-Load CPS variables for Quick Start */
 CPSON=0;			// InActivate CPS module
for(current_ch=0; current_ch < NOFCH; current_ch++)	
	{
	CPSCON1	=current_ch;	// Select the channel of CPS
 	TMR1H =TMR1L =0x00;		// Timer1NA
 	CPSON=1;				// Activate CPS module
 	__delay_us(1030);		// Wait for Measurement duration
 	CPSON =0;				// Inactive(Stop) CPS module (count)
							// Set initial value for speed-up
 	SUM_SHORT[current_ch] = 4 *(TMR1H*512+TMR1L);
	SUM_LONG[current_ch] =512 * (unsigned long)SUM_SHORT[current_ch];	
	}

 current_ch = scanning_cps =0;			// Start from channel 0 CPS
 TMR1H =TMR1L =TMR0 =0x00;	// Timer1lNA (prepare for the next time)
 CPSON=1;					// Activate CPS module
 PEIE =1;					// Enable Embedded Peripheralls inturrupt
 GIE=1;						// 荞݋ (GIE as INTCON bit7)

while(1)	// [v@ĩ[vtx_data[]ɃZbgĂlς΃RR[h蓖Ă̕ύX\j
	{	
	if((SUM_LONG[current_ch]/520) > (SUM_SHORT[current_ch]))	//ԈړςZړς̃JEgl1.5%ȏ㏬ȂȂ
		{														//Ódeʂ債@Ă
		switch( current_ch )									//ꂽCPSԍʂ̏
			{
			case 3:
				//ALPINE PRIVOUS
				Alpine4byte();	//as set toshiba mode
				tx_data[0]=0x86;
				tx_data[1]=0x72;
				tx_data[2]=0x13;
				tx_data[3]=0xec;
				GIE=0;							// 荞ݕs (GIE as INTCON bit7)
				transmit();	//IR transmition
    			TMR0IF =TMR0 =TMR1H =TMR1L =0;	// Timer0, Timer1NA (prepare for the next time) and Clear the inturrupt flag
   				TMR0IE =CPSON =1;				// Restart CPS module count also Timer1 ISR enable
				GIE=1;							// 荞݋ (GIE as INTCON bit7)
				__delay_ms(140);
				break;

			case 2:
				//ALPINE NEXT
				Alpine4byte();	//as set toshiba mode
				tx_data[0]=0x86;
				tx_data[1]=0x72;
				tx_data[2]=0x12;
				tx_data[3]=0xed;
				GIE=0;				// 荞ݕs (GIE as INTCON bit7)
				transmit();	//IR transmition
	   			TMR0IF =TMR0 =TMR1H =TMR1L =0;	// Timer0, Timer1NA (prepare for the next time) and Clear the inturrupt flag
   				TMR0IE =CPSON =1;				// Restart CPS module count also Timer1 ISR enable
				GIE=1;							// 荞݋ (GIE as INTCON bit7)
				__delay_ms(140);
				break;

			case 1:
				//ALPINE VOL-UP
				Alpine4byte();	//as set toshiba mode
				tx_data[0]=0x86;
				tx_data[1]=0x72;
				tx_data[2]=0x14;
				tx_data[3]=0xeB;
				transmit();	//IR transmition
    			TMR0IF =TMR0 =TMR1H =TMR1L =0;	// Timer0, Timer1NA (prepare for the next time) and Clear the inturrupt flag
   				TMR0IE =CPSON =1;				// Restart CPS module count also Timer1 ISR enable
				__delay_ms(140);
				GIE=1;							// 荞݋ (GIE as INTCON bit7)
				break;

			case 0:
			default:
				//ALPINE VOL DOWN
				Alpine4byte();	//as set toshiba mode
				tx_data[0]=0x86;
				tx_data[1]=0x72;
				tx_data[2]=0x15;
				tx_data[3]=0xEA;
				GIE=0;							// 荞ݕs (GIE as INTCON bit7)
				transmit();	//IR transmition
    			TMR0IF =TMR0 =TMR1H =TMR1L =0;	// Timer0, Timer1NA (prepare for the next time) and Clear the inturrupt flag
   				TMR0IE =CPSON =1;				// Restart CPS module count also Timer1 ISR enable
				__delay_ms(140);
				GIE=1;							// 荞݋ (GIE as INTCON bit7)
			}
        }
 	current_ch = (++current_ch) % NOFCH;		//@ꂩ珈CPSW[̔ԍvZ
   }
}


/*
----< Timer0 Inturrupt Function >----
*/

static void interrupt ISR(void)
{
static unsigned long work;

if(TMR0IF==1)				
	{					//----< Timer0 overflown Inturrupt@process >----	
	TMR0IE = CPSON=0;	// Stop CPS module's counting
						// Use Long to Prevent from caliculation overflown error
	work = SUM_SHORT[scanning_cps] =(SUM_SHORT[scanning_cps] - (SUM_SHORT[scanning_cps]/4)) + ((unsigned int)TMR1H * 256) + (unsigned int)TMR1L;
						// ZԈړς̌vZiߋ4̑al1/4̑ɍ肵lgݓj
	SUM_LONG[scanning_cps] = (SUM_LONG[scanning_cps] - SUM_LONG[scanning_cps]/512) + work;
						//@512̒Ԉړρiߋ512̑al1/512̑ɍ肵lgݓj	
 	scanning_cps =(++scanning_cps) % NOFCH;	// Calc Next CPS channel number (Cyclic from CPS_0 to PCS_N)
	CPSCON1	=0b00000111 & scanning_cps;		// Select the channel of CPS
    TMR0IF =TMR0 =TMR1H =TMR1L =0;			// Timer0, Timer1NA (prepare for the next time) and Clear the inturrupt flag
    TMR0IE =CPSON =1;						// Restart CPS module count also Timer1 ISR enable
    }
return;	// Return form ISR
}

//AiOMUXɂ钍ӎi26.1͂j
//W[LĂԂCPSxsI悤Ƃ
//CPSxs̐ÓdeʌoIV^[ƂȂANSELTRISrbg
//̃ZbgɎsU~gǂ߂ȂȂ邱Ƃ܂B